@@ -34,9 +34,17 @@ static HandlerResult AffinityPanel_eventHandler(Panel* this, int ch) {
    return IGNORED;
 }
 
+PanelClass AffinityPanel_class = {
+   .super = {
+      .extends = Class(Panel),
+      .delete = Panel_delete
+   },
+   .eventHandler = AffinityPanel_eventHandler
+};
+
 Panel* AffinityPanel_new(ProcessList* pl, Affinity* affinity) {
-   Panel* this = Panel_new(1, 1, 1, 1, CHECKITEM_CLASS, true, ListItem_compare);
-   this->eventHandler = AffinityPanel_eventHandler;
+   Panel* this = Panel_new(1, 1, 1, 1, true, Class(CheckItem));
+   Object_setClass(this, Class(AffinityPanel));
 
    Panel_setHeader(this, "Use CPUs:");
    int curCpu = 0;
@@ -50,7 +58,7 @@ Panel* AffinityPanel_new(ProcessList* pl, Affinity* affinity) {
       } else {
          mode = false;
       }
-      Panel_add(this, (Object*) CheckItem_new(strdup(number), NULL, mode));         
+      Panel_add(this, (Object*) CheckItem_new(strdup(number), NULL, mode));
    }
    return this;
 }
