@@ -37,7 +37,7 @@ static void AvailableMetersPanel_delete(Object* object) {
    free(this);
 }
 
-static inline void AvailableMetersPanel_addHeader(Header* header, Panel* panel, MeterType* type, int param, HeaderSide side) {
+static inline void AvailableMetersPanel_addHeader(Header* header, Panel* panel, MeterClass* type, int param, HeaderSide side) {
    Meter* meter = (Meter*) Header_addMeter(header, type, param, side);
    Panel_add(panel, (Object*) Meter_toListItem(meter));
 }
@@ -78,26 +78,32 @@ static HandlerResult AvailableMetersPanel_eventHandler(Panel* super, int ch) {
    return result;
 }
 
+PanelClass AvailableMetersPanel_class = {
+   .super = {
+      .extends = Class(Panel),
+      .delete = AvailableMetersPanel_delete
+   },
+   .eventHandler = AvailableMetersPanel_eventHandler
+};
+
 AvailableMetersPanel* AvailableMetersPanel_new(Settings* settings, Panel* leftMeters, Panel* rightMeters, ScreenManager* scr) {
-   AvailableMetersPanel* this = (AvailableMetersPanel*) malloc(sizeof(AvailableMetersPanel));
+   AvailableMetersPanel* this = AllocThis(AvailableMetersPanel);
    Panel* super = (Panel*) this;
-   Panel_init(super, 1, 1, 1, 1, LISTITEM_CLASS, true);
-   ((Object*)this)->delete = AvailableMetersPanel_delete;
+   Panel_init(super, 1, 1, 1, 1, Class(ListItem), true);
    
    this->settings = settings;
    this->leftPanel = leftMeters;
    this->rightPanel = rightMeters;
    this->scr = scr;
-   super->eventHandler = AvailableMetersPanel_eventHandler;
 
    Panel_setHeader(super, "Available meters");
    for (int i = 1; Meter_types[i]; i++) {
-      MeterType* type = Meter_types[i];
-      if (type != &CPUMeter) {
+      MeterClass* type = Meter_types[i];
+      if (type != &CPUMeter_class) {
          Panel_add(super, (Object*) ListItem_new(type->uiName, i << 16));
       }
    }
-   MeterType* type = &CPUMeter;
+   MeterClass* type = &CPUMeter_class;
    int cpus = settings->pl->cpuCount;
    if (cpus > 1) {
       Panel_add(super, (Object*) ListItem_new("CPU average", 0));
