@@ -101,6 +101,12 @@ static HandlerResult CategoriesPanel_eventHandler(Panel* super, int ch) {
             result = HANDLED;
          break;
       }
+      default:
+         if (isalpha(ch))
+            result = Panel_selectByTyping(super, ch);
+         if (result == BREAK_LOOP)
+            result = IGNORED;
+         break;
    }
 
    if (result == HANDLED) {
@@ -126,15 +132,21 @@ static HandlerResult CategoriesPanel_eventHandler(Panel* super, int ch) {
    return result;
 }
 
+PanelClass CategoriesPanel_class = {
+   .super = {
+      .extends = Class(Panel),
+      .delete = CategoriesPanel_delete
+   },
+   .eventHandler = CategoriesPanel_eventHandler
+};
+
 CategoriesPanel* CategoriesPanel_new(Settings* settings, ScreenManager* scr) {
-   CategoriesPanel* this = (CategoriesPanel*) malloc(sizeof(CategoriesPanel));
+   CategoriesPanel* this = AllocThis(CategoriesPanel);
    Panel* super = (Panel*) this;
-   Panel_init(super, 1, 1, 1, 1, LISTITEM_CLASS, true);
-   ((Object*)this)->delete = CategoriesPanel_delete;
+   Panel_init(super, 1, 1, 1, 1, Class(ListItem), true);
 
    this->settings = settings;
    this->scr = scr;
-   super->eventHandler = CategoriesPanel_eventHandler;
    Panel_setHeader(super, "Setup");
    Panel_add(super, (Object*) ListItem_new("Meters", 0));
    Panel_add(super, (Object*) ListItem_new("Display options", 0));
