@@ -1,5 +1,5 @@
 /*
-htop
+htop - Hashtable.c
 (C) 2004-2011 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
@@ -8,12 +8,11 @@ in the source distribution for its full text.
 #include "Hashtable.h"
 
 #include <stdlib.h>
-#include <stdbool.h>
 #include <assert.h>
 
-#include "debug.h"
-
 /*{
+#include <stdbool.h>
+
 typedef struct Hashtable_ Hashtable;
 
 typedef void(*Hashtable_PairFunction)(int, void*, void*);
@@ -77,7 +76,7 @@ Hashtable* Hashtable_new(int size, bool owner) {
    this = (Hashtable*) malloc(sizeof(Hashtable));
    this->items = 0;
    this->size = size;
-   this->buckets = (HashtableItem**) calloc(sizeof(HashtableItem*), size);
+   this->buckets = (HashtableItem**) calloc(size, sizeof(HashtableItem*));
    this->owner = owner;
    assert(Hashtable_isConsistent(this));
    return this;
