@@ -11,7 +11,6 @@ in the source distribution for its full text.
 #include "ProcessList.h"
 
 #include <stdlib.h>
-#include <curses.h>
 #include <string.h>
 #include <math.h>
 #include <sys/param.h>
@@ -53,17 +52,21 @@ static void MemoryMeter_display(Object* cast, RichString* out) {
    RichString_append(out, CRT_colors[MEMORY_USED], buffer);
    sprintf(buffer, format, buffersMem);
    RichString_append(out, CRT_colors[METER_TEXT], "buffers:");
-   RichString_append(out, CRT_colors[MEMORY_BUFFERS], buffer);
+   RichString_append(out, CRT_colors[MEMORY_BUFFERS_TEXT], buffer);
    sprintf(buffer, format, cachedMem);
    RichString_append(out, CRT_colors[METER_TEXT], "cache:");
    RichString_append(out, CRT_colors[MEMORY_CACHE], buffer);
 }
 
-MeterType MemoryMeter = {
+MeterClass MemoryMeter_class = {
+   .super = {
+      .extends = Class(Meter),
+      .delete = Meter_delete,
+      .display = MemoryMeter_display,
+   },
    .setValues = MemoryMeter_setValues, 
-   .display = MemoryMeter_display,
-   .mode = BAR_METERMODE,
-   .items = 3,
+   .defaultMode = BAR_METERMODE,
+   .maxItems = 3,
    .total = 100.0,
    .attributes = MemoryMeter_attributes,
    "Memory",
