@@ -6,23 +6,29 @@ in the source distribution for its full text.
 */
 
 #include "Settings.h"
-#include "String.h"
-#include "ProcessList.h"
-#include "Header.h"
 
-#include "debug.h"
+#include "String.h"
+#include "Vector.h"
+
+#include <sys/stat.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
 
 #define DEFAULT_DELAY 15
 
 /*{
+#include "ProcessList.h"
+#include "Header.h"
+#include <stdbool.h>
 
 typedef struct Settings_ {
    char* userSettings;
    ProcessList* pl;
    Header* header;
    int colorScheme;
-   bool changed;
    int delay;
+   bool changed;
 } Settings;
 
 }*/
@@ -55,14 +61,11 @@ static void Settings_readMeterModes(Settings* this, char* line, HeaderSide side)
    String_freeArray(ids);
 }
 
-static bool Settings_read(Settings* this, char* fileName, int cpuCount) {
-   // TODO: implement File object and make
-   // file I/O object-oriented.
-   FILE* fd;
-   fd = fopen(fileName, "r");
-   if (fd == NULL) {
+static bool Settings_read(Settings* this, const char* fileName, int cpuCount) {
+   FILE* fd = fopen(fileName, "r");
+   if (!fd)
       return false;
-   }
+   
    const int maxLine = 2048;
    char buffer[maxLine];
    bool readMeters = false;
@@ -79,11 +82,13 @@ static bool Settings_read(Settings* this, char* fileName, int cpuCount) {
          char** ids = String_split(trim, ' ', &nIds);
          free(trim);
          int i, j;
+         this->pl->flags = 0;
          for (j = 0, i = 0; i < LAST_PROCESSFIELD && ids[i]; i++) {
             // This "+1" is for compatibility with the older enum format.
             int id = atoi(ids[i]) + 1;
             if (id > 0 && id < LAST_PROCESSFIELD) {
                this->pl->fields[j] = id;
+               this->pl->flags |= Process_fieldFlags[id];
                j++;
             }
          }
@@ -121,6 +126,10 @@ static bool Settings_read(Settings* this, char* fileName, int cpuCount) {
          this->pl->detailedCPUTime = atoi(option[1]);
       } else if (String_eq(option[0], "cpu_count_from_zero")) {
          this->pl->countCPUsFromZero = atoi(option[1]);
+      } else if (String_eq(option[0], "update_process_names")) {
+         this->pl->updateProcessNames = atoi(option[1]);
+      } else if (String_eq(option[0], "account_guest_in_cpu_meter")) {
+         this->pl->accountGuestInCPUMeter = atoi(option[1]);
       } else if (String_eq(option[0], "delay")) {
          this->delay = atoi(option[1]);
       } else if (String_eq(option[0], "color_scheme")) {
@@ -180,6 +189,8 @@ bool Settings_write(Settings* this) {
    fprintf(fd, "header_margin=%d\n", (int) this->header->margin);
    fprintf(fd, "detailed_cpu_time=%d\n", (int) this->pl->detailedCPUTime);
    fprintf(fd, "cpu_count_from_zero=%d\n", (int) this->pl->countCPUsFromZero);
+   fprintf(fd, "update_process_names=%d\n", (int) this->pl->updateProcessNames);
+   fprintf(fd, "account_guest_in_cpu_meter=%d\n", (int) this->pl->accountGuestInCPUMeter);
    fprintf(fd, "color_scheme=%d\n", (int) this->colorScheme);
    fprintf(fd, "delay=%d\n", (int) this->delay);
    fprintf(fd, "left_meters=");
@@ -212,21 +223,50 @@ Settings* Settings_new(ProcessList* pl, Header* header, int cpuCount) {
    Settings* this = malloc(sizeof(Settings));
    this->pl = pl;
    this->header = header;
-   const char* home;
-   char* rcfile;
-   home = getenv("HOME_ETC");
-   if (!home) home = getenv("HOME");
-   if (!home) home = "";
-   rcfile = getenv("HOMERC");
-   if (!rcfile)
-      this->userSettings = String_cat(home, "/.htoprc");
-   else
-      this->userSettings = String_copy(rcfile);
+   char* legacyDotfile = NULL;
+   char* rcfile = getenv("HTOPRC");
+   if (rcfile) {
+      this->userSettings = strdup(rcfile);
+   } else {
+      const char* home = getenv("HOME");
+      if (!home) home = "";
+      const char* xdgConfigHome = getenv("XDG_CONFIG_HOME");
+      char* configDir = NULL;
+      char* htopDir = NULL;
+      if (xdgConfigHome) {
+         this->userSettings = String_cat(xdgConfigHome, "/htop/htoprc");
+         configDir = strdup(xdgConfigHome);
+         htopDir = String_cat(xdgConfigHome, "/htop");
+      } else {
+         this->userSettings = String_cat(home, "/.config/htop/htoprc");
+         configDir = String_cat(home, "/.config");
+         htopDir = String_cat(home, "/.config/htop");
+      }
+      legacyDotfile = String_cat(home, "/.htoprc");
+      (void) mkdir(configDir, 0700);
+      (void) mkdir(htopDir, 0700);
+      free(htopDir);
+      free(configDir);
+      struct stat st;
+      if (lstat(legacyDotfile, &st) != 0) {
+         st.st_mode = 0;
+      }
+      if (access(legacyDotfile, R_OK) != 0 || S_ISLNK(st.st_mode)) {
+         free(legacyDotfile);
+         legacyDotfile = NULL;
+      }
+   }
    this->colorScheme = 0;
    this->changed = false;
    this->delay = DEFAULT_DELAY;
-   bool ok = Settings_read(this, this->userSettings, cpuCount);
-   if (!ok) {
+   bool ok = Settings_read(this, legacyDotfile ? legacyDotfile : this->userSettings, cpuCount);
+   if (ok) {
+      if (legacyDotfile) {
+         // Transition to new location and delete old configuration file
+         if (Settings_write(this))
+            unlink(legacyDotfile);
+      }
+   } else {
       this->changed = true;
       // TODO: how to get SYSCONFDIR correctly through Autoconf?
       char* systemSettings = String_cat(SYSCONFDIR, "/htoprc");
@@ -239,5 +279,6 @@ Settings* Settings_new(ProcessList* pl, Header* header, int cpuCount) {
          pl->highlightThreads = false;
       }
    }
+   free(legacyDotfile);
    return this;
 }
